@@ -1,8 +1,7 @@
 use VRML::VRML1::Standard;
 
 $vrml = new VRML::VRML1::Standard;
-$vrml->VRML_head("#VRML V1.0 ascii")
-->Separator('Demo Cube')
+$vrml->Separator('Demo Cube')
 ->Cube(5,3,2)
 ->End
-->print;
+->save;
